----------------------------------------------------------------------------------
-- POLYTECHNIQUE MONTREAL
-- ELE3311 - Systemes logiques programmables 
-- 
-- Module Name:    msa_reg 
-- Description: 
--
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with signed or unsigned values
--use IEEE.numeric_std.all;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity msa_reg is
port (
  rst_i                 : in    std_logic;
  clk_i                 : in    std_logic;
  next_i                : in    std_logic;
  addr_cnt_rst_o        : out   std_logic;
  addr_cnt_inc_o        : out   std_logic
);
end msa_reg;


architecture behavioral of msa_reg is

  type t_etat is (
    reset,
    etat_a,
    etat_b,
    etat_c
  );

  signal rst                   : std_logic;
  signal clk                   : std_logic;
  -- Registered internal signals for outputs
  signal addr_cnt_rst_p        : std_logic;
  signal addr_cnt_inc_p        : std_logic;

  signal addr_cnt_rst_f        : std_logic;
  signal addr_cnt_inc_f        : std_logic;

  -- Internal signals
  signal etat_p, etat_f        : t_etat;

begin
  rst <= rst_i;
  clk <= clk_i;

  REGISTERED: process(clk)
  begin
    if (rst = '1') then
        etat_p <= reset;
        addr_cnt_rst_p <= '1';
        addr_cnt_inc_p <= '0';
    elsif (rising_edge(clk)) then
        etat_p <= etat_f;
        addr_cnt_rst_p <= addr_cnt_rst_f;
        addr_cnt_inc_p <= addr_cnt_inc_f;
    end if;
    end process;


  COMBINATORIAL: process(next_i, etat_p, addr_cnt_rst_p)
  begin
    case etat_p is
        when reset =>
            addr_cnt_rst_f <= '1';
            addr_cnt_inc_f <= '0';
            etat_f <= etat_a;

        when etat_a =>
            addr_cnt_rst_f <= '0';
            addr_cnt_inc_f <= '0';
            if (next_i = '0') then
                etat_f <= etat_a;
            else
                etat_f <= etat_b;
            end if;

        when etat_b =>
            addr_cnt_rst_f <= '0';
            addr_cnt_inc_f <= '1';
            etat_f <= etat_c;

        when etat_c =>
            addr_cnt_rst_f <= '0';
            addr_cnt_inc_f <= '0';
            if (next_i = '1') then
                etat_f <= etat_c;
            else
                etat_f <= etat_a;
            end if;
    end case;

  end process;


  ----------------------------------------------------------------------------
  -- Assign outputs
  ----------------------------------------------------------------------------
  -- *** A completer ***
  addr_cnt_rst_o <= addr_cnt_rst_p;
  addr_cnt_inc_o <= addr_cnt_inc_p;

end behavioral;
