----------------------------------------------------------------------------------
-- POLYTECHNIQUE MONTREAL
-- ELE3311 - Systemes logiques programmables 
-- 
-- Module Name:    Pulse Width Modulation (PWM) Wave Generator
-- Description:    Registre de ratio suivi d'un comparateur
--                 Le compteur en boucle est partage
--
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with signed or unsigned values
use IEEE.numeric_std.all;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity pwm is
generic (
  PRECISION             : integer := 8
);
port (
  rst_i                 : in    std_logic;  -- synchrone
  clk_i                 : in    std_logic;
  wr_en_i               : in    std_logic;  -- ecriture du ratio dans le registre
  ratio_i               : in    std_logic_vector(PRECISION-1 downto 0);
  loop_cnt_i            : in    std_logic_vector(PRECISION-1 downto 0);
  pwm_heat_o            : out   std_logic
);
end pwm;


architecture behavioral of pwm is

  signal clk                  : std_logic;

  signal ratio_reg_f          : std_logic_vector(PRECISION-1 downto 0);
  signal pwm_heat_f           : std_logic;

  signal ratio_reg_p          : std_logic_vector(PRECISION-1 downto 0);
  signal pwm_heat_p           : std_logic;

begin
  clk <= clk_i;
  ratio_reg_f <= ratio_i;


  REGISTERED: process(clk, wr_en_i, ratio_reg_f)
  begin
    if rising_edge(clk) then
    -- *** A completer ***
      if(wr_en_i = '1') then
        ratio_reg_p <= ratio_reg_f;
      end if;

      pwm_heat_p <= pwm_heat_f;

    end if;
  end process;


  ---------------------------------------------------------------------------
  -- Section combinatoire avec un processus ou
  -- des assignations conditionnelles

  -- *** A completer ***
  COMBINATORIAL: process(loop_cnt_i, ratio_reg_p)
  begin
    if (loop_cnt_i < ratio_reg_p) then
      pwm_heat_f <= '1';
    else
      pwm_heat_f <= '0';
    end if;
    -- pwm_heat_f <= (loop_cnt_i < ratio_reg_p);
  end process;

  -- Assign outputs
  -- *** A completer ***
  pwm_heat_o <= pwm_heat_p;

end behavioral;
