-- Banc d'essai avec verifiation automatique des sorties.
-- Les descriptions de ce fichier sont majoritairement non-synthetisables.
-- Self-Checking Testbench (SCTB)

-- Mise a jour (synchroniser avec TB_VERSION):
-- . 2021-02-10 : Creation
--
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
 
entity ctrl_temp_del_sctb is
generic (
  TB_VERSION              : string := "v2020-02-10";
  SHORT_SIM               : boolean := true
);
end ctrl_temp_del_sctb;
 
architecture testbench of ctrl_temp_del_sctb is 

  -- component declaration for the unit under test (uut)

  component ctrl_temp_del
  generic (
    SHORT_SIM             : boolean := false;
    HEATERS               : integer := 4;
    PRECISION             : integer := 8
  );
  port (
    --RST & CLK
    rst_n_i               : in    std_logic;
    clk_i                 : in    std_logic;
        
    prog_i                : in    std_logic;
    next_i                : in    std_logic;
    btn_i                 : in    std_logic_vector(2 downto 0);
    ratio_i               : in    std_logic_vector(7 downto 0);
  
    del_o                 : out   std_logic_vector(7 downto 0)
  );
  end component;
        
  -- Detection de la sequence
  type detect_t is (IDLE);
  signal detect                   : detect_t := IDLE;
  signal detect_time              : time := 0 ns;

  -- Entrees
  signal pb_prog                  : std_logic := '0';
  signal pb_next                  : std_logic := '0';
  signal clk                      : std_logic := '0';
  signal rst                      : std_logic := '1';
  signal rst_n                    : std_logic;
  signal ratio                    : std_logic_vector(7 downto 0) := X"83";

  -- Sorties
  signal del                      : std_logic_vector(7 downto 0);

  -- Periode d'horloge
  constant clk_period             : time := 10 ns; -- 100 MHz

  -- Verification
  signal clk_en                   : boolean := true;

  procedure set_btns (
    variable sel_prog     : in  boolean;
    variable sel_next     : in  boolean;
    variable nb_clk_high  : in  integer;
    variable nb_clk_low   : in  integer;
    signal pb_prog        : out std_logic;
    signal pb_next        : out std_logic
  ) is
  begin
    if (sel_prog) then 
      pb_prog             <= '1';
    end if;
    if (sel_next) then 
      pb_next             <= '1';
    end if;

    for i in 0 to nb_clk_high loop
      wait until falling_edge(clk);
    end loop;

    pb_prog             <= '0';
    pb_next             <= '0';

    if (nb_clk_low > 0) then
      for i in 0 to nb_clk_low loop
        wait until falling_edge(clk);
      end loop;
    end if;


  end set_btns;

begin

  -- Instantiation du composant a verifier - Unit Under Test (UUT)
  UUT: ctrl_temp_del 
  generic map (
    SHORT_SIM             => SHORT_SIM,
    HEATERS               => 4,
    PRECISION             => 8
  )
  port map (
    rst_n_i               => rst_n,
    clk_i                 => clk,
    prog_i                => pb_prog,
    next_i                => pb_next,
    btn_i                 => "000",
    ratio_i               => ratio,
    del_o                 => del
  );

  -- Generation de l'horloge et de la remise a zero
  clk    <= not clk after clk_period/2 when clk_en else
            '0';

  rst    <= '1', '0' after clk_period*10;
  rst_n  <= not rst;


  -- Stimulus process
  driver: process
    constant nb_clk_1_ms              : integer := 1000;
    variable sel_prog                 : boolean := false;
    variable sel_next                 : boolean := false;
    variable nb_clk_high              : integer := 2 * nb_clk_1_ms;
    variable nb_clk_low               : integer := 3 * nb_clk_1_ms;
  begin
    -- On synchronise les signaux avec le front descendant de l'horloge
    -- afin de respecter les temps de preparation et de maintien (setup/hold times) 
    for i in 0 to 1000 loop
      wait until falling_edge(clk);
    end loop;

    -- nb_clk_high  := 9 * nb_clk_1_ms;
    -- nb_clk_low   := 15 * nb_clk_1_ms;

    report "***** Section a inclure au rapport (DEBUT) " & TB_VERSION & " *****";
    --------------------------------------------------------------------------
    report "Programmation du PWM 0 a 0";
    --------------------------------------------------------------------------
    ratio       <= std_logic_vector(to_unsigned(0, ratio'length));
    sel_prog    := true;
    set_btns(
      sel_prog => sel_prog, sel_next => sel_next,
      nb_clk_high => nb_clk_high, nb_clk_low => nb_clk_low,
      pb_prog => pb_prog, pb_next => pb_next
    );
    sel_prog    := false;
    sel_next    := true;
    set_btns(
      sel_prog => sel_prog, sel_next => sel_next,
      nb_clk_high => nb_clk_high, nb_clk_low => nb_clk_low,
      pb_prog => pb_prog, pb_next => pb_next
    );
    sel_next    := false;

    --------------------------------------------------------------------------
    report "Programmation du PWM 1 a 1";
    --------------------------------------------------------------------------
    ratio       <= std_logic_vector(to_unsigned(1, ratio'length));
    sel_prog    := true;
    set_btns(
      sel_prog => sel_prog, sel_next => sel_next,
      nb_clk_high => nb_clk_high, nb_clk_low => nb_clk_low,
      pb_prog => pb_prog, pb_next => pb_next
    );
    sel_prog    := false;
    sel_next    := true;
    set_btns(
      sel_prog => sel_prog, sel_next => sel_next,
      nb_clk_high => nb_clk_high, nb_clk_low => nb_clk_low,
      pb_prog => pb_prog, pb_next => pb_next
    );
    sel_next    := false;

    --------------------------------------------------------------------------
    report "Programmation du PWM 2 a 255";
    --------------------------------------------------------------------------
    ratio       <= std_logic_vector(to_unsigned(255, ratio'length));
    sel_prog    := true;
    set_btns(
      sel_prog => sel_prog, sel_next => sel_next,
      nb_clk_high => nb_clk_high, nb_clk_low => nb_clk_low,
      pb_prog => pb_prog, pb_next => pb_next
    );
    sel_prog    := false;
    sel_next    := true;
    set_btns(
      sel_prog => sel_prog, sel_next => sel_next,
      nb_clk_high => nb_clk_high, nb_clk_low => nb_clk_low,
      pb_prog => pb_prog, pb_next => pb_next
    );
    sel_next    := false;

    --------------------------------------------------------------------------
    report "Programmation du PWM 3 a 142";
    --------------------------------------------------------------------------
    ratio       <= std_logic_vector(to_unsigned(142, ratio'length));
    sel_prog    := true;
    set_btns(
      sel_prog => sel_prog, sel_next => sel_next,
      nb_clk_high => nb_clk_high, nb_clk_low => nb_clk_low,
      pb_prog => pb_prog, pb_next => pb_next
    );
    sel_prog    := false;
    sel_next    := true;
    set_btns(
      sel_prog => sel_prog, sel_next => sel_next,
      nb_clk_high => nb_clk_high, nb_clk_low => nb_clk_low,
      pb_prog => pb_prog, pb_next => pb_next
    );
    sel_next    := false;

    --------------------------------------------------------------------------
    report "Programmation du PWM 0 a 66 (boucle du compteur d'adresse)";
    --------------------------------------------------------------------------
    ratio       <= std_logic_vector(to_unsigned(255, ratio'length));
    sel_prog    := true;
    set_btns(
      sel_prog => sel_prog, sel_next => sel_next,
      nb_clk_high => nb_clk_high, nb_clk_low => nb_clk_low,
      pb_prog => pb_prog, pb_next => pb_next
    );
    sel_prog    := false;

  -- TODO Ajout de cas limite


  for i in 0 to 12000 loop
      wait until falling_edge(clk);
    end loop;

    report "***** Section a inclure au rapport (FIN) *****";

    -- Fin de la sequence de test
    -- Desactivation de l'horloge
    clk_en    <= false;
    wait;
   end process;

  --*************************************************************************
  --  Verification
  --*************************************************************************
  -- output_monitor: process(pb_prog, pb_next)
  --   -- Utilisation de variables pour une valeur a jour en meme temps
  --   -- que la transition et eviter les delta-delai (les pas de simulation)
  --   variable or_tbd           : std_logic;
  -- begin

  --   case detect is
  --     when IDLE =>
  --     when others =>
  --       detect        <= IDLE;
  --   end case;
  -- end process;

  -- output_assert: process(detect, dels)
  --   variable detect_time      : time;
  -- begin
  -- end process;

end testbench;